/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.Vector;
import EDU.auburn.VGJ.graph.Node;
import EDU.utwente.CoCompose.lang.*;
import EDU.utwente.CoCompose.util.*;

/**
 * A dialog window for editing the properties of Concepts, Composites and
 * CompositeRoles.
 *
 * @author Dennis Wagelaar
 * @version 0.1.18, 24 November 2003
 */

public class ConceptPropertiesDialog extends CoComposeDialog
  implements FilenameFilter
{
  //////////////////////////////////////////////////////////////////////////////
  // Constants
  //////////////////////////////////////////////////////////////////////////////

  private static final String CONSTRAINTS_DIR = "plugins/classes/constraints";

  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  private Frame frame_;
  private AbstractConcept concept_;
  private JPanel addToPanel_;
  private Vector partOf_ = new Vector();
  JPanel mainPanel = new JPanel();
  JPanel typePanel = new JPanel();
  JPanel boundsPanel1 = new JPanel();
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel propertiesPanel = new JPanel();
  JPanel buttonPanel = new JPanel();
  JButton okBtn = new JButton();
  JButton cancelBtn = new JButton();
  FlowLayout flowLayout1 = new FlowLayout();
  JLabel jLabel1 = new JLabel();
  JLabel jLabel2 = new JLabel();
  JLabel jLabel3 = new JLabel();
  JLabel jLabel4 = new JLabel();
  JPanel namePanel = new JPanel();
  FlowLayout flowLayout3 = new FlowLayout();
  JPanel commentsPanel = new JPanel();
  FlowLayout flowLayout4 = new FlowLayout();
  JPanel specificationPanel = new JPanel();
  FlowLayout flowLayout5 = new FlowLayout();
  JPanel specificationLangPanel = new JPanel();
  FlowLayout flowLayout6 = new FlowLayout();
  JPanel jPanel1 = new JPanel();
  JPanel jPanel2 = new JPanel();
  BorderLayout borderLayout2 = new BorderLayout();
  BorderLayout borderLayout3 = new BorderLayout();
  BorderLayout borderLayout4 = new BorderLayout();
  JPanel jPanel3 = new JPanel();
  BorderLayout borderLayout5 = new BorderLayout();
  FlowLayout flowLayout7 = new FlowLayout();
  JLabel jLabel5 = new JLabel();
  JLabel jLabel6 = new JLabel();
  JLabel jLabel7 = new JLabel();
  JPanel constraintPanel = new JPanel();
  BorderLayout borderLayout6 = new BorderLayout();
  JPanel correctnessPanel = new JPanel();
  JLabel jLabel8 = new JLabel();
  FlowLayout flowLayout8 = new FlowLayout();
  JPanel constraintPanel1 = new JPanel();
  JPanel completenessPanel = new JPanel();
  BorderLayout borderLayout7 = new BorderLayout();
  JLabel jLabel9 = new JLabel();
  FlowLayout flowLayout10 = new FlowLayout();
  BorderLayout borderLayout8 = new BorderLayout();
  JPanel boundsPanel = new JPanel();
  BorderLayout borderLayout10 = new BorderLayout();
  JPanel typePanel1 = new JPanel();
  BorderLayout borderLayout11 = new BorderLayout();
  JPanel iconOnlyPanel = new JPanel();
  FlowLayout flowLayout2 = new FlowLayout();
  JPanel typePanel2 = new JPanel();
  BorderLayout borderLayout14 = new BorderLayout();
  FlowLayout flowLayout12 = new FlowLayout();
  JPanel compositeButtonPanel = new JPanel();
  FlowLayout flowLayout13 = new FlowLayout();
  JButton solutionsBtn = new JButton();
  JTextField nameEdit = new JTextField();
  JScrollPane jScrollPane1 = new JScrollPane();
  JTextArea commentsEdit = new JTextArea();
  JScrollPane jScrollPane2 = new JScrollPane();
  JTextArea specificationEdit = new JTextArea();
  JTextField specificationLangEdit = new JTextField();
  JTextField fromRealmEdit = new JTextField();
  JTextField typeEdit = new JTextField();
  JComboBox correctnessBox = new JComboBox();
  JComboBox completenessBox = new JComboBox();
  JComboBox lowerBoundBox = new JComboBox();
  JComboBox upperBoundBox = new JComboBox();
  JCheckBox iconOnlyCheckBox = new JCheckBox();
  JPanel typePanel3 = new JPanel();
  BorderLayout borderLayout12 = new BorderLayout();
  JPanel implementationsPanel = new JPanel();
  FlowLayout flowLayout14 = new FlowLayout();
  JButton implementationsBtn = new JButton();
  JPanel implementationsPanel1 = new JPanel();
  BorderLayout borderLayout13 = new BorderLayout();
  JButton implPattButton = new JButton();
  JPanel partOfPanel = new JPanel();
  JComboBox partOfBox = new JComboBox();
  JLabel jLabel11 = new JLabel();
  JLabel jLabel12 = new JLabel();
  BorderLayout borderLayout9 = new BorderLayout();
  FlowLayout flowLayout9 = new FlowLayout();
  JPanel lastPanel = new JPanel();
  FlowLayout flowLayout11 = new FlowLayout();
  JPanel publicNamePanel = new JPanel();
  JTextField publicNameEdit = new JTextField();
  JLabel jLabel10 = new JLabel();
  JPanel realmPanel = new JPanel();
  JPanel realmPanel1 = new JPanel();

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a ConceptPropertiesDialog for a Concept object.
   *
   * @param frame The parent frame.
   * @param title The dialog title.
   * @param modal Shows the dialog in modal mode when true.
   * @param concept The Concept of which the properties will be edited.
   */
  public ConceptPropertiesDialog(Frame frame, String title, boolean modal,
    AbstractConcept concept)
  {
    super(frame, title, modal);
    frame_ = frame;
    concept_ = concept;
    try
    {
      jbInit();
      mInit();
      pack();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  } //ConceptPropertiesDialog(Frame frame, String title, boolean modal, Concept concept)

  /**
   * Constructs a ConceptPropertiesDialog.
   * This constructor is only for the JBuilder Designer, do not use it!
   */
  public ConceptPropertiesDialog()
  {
    this(null, "", false, null);
  } //ConceptPropertiesDialog()

  //////////////////////////////////////////////////////////////////////////////
  // Private methods
  //////////////////////////////////////////////////////////////////////////////

  /* JBuilder Component initialization. */
  void jbInit() throws Exception
  {
    fromRealmEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    fromRealmEdit.setPreferredSize(new Dimension(400, 21));
    fromRealmEdit.setEditable(false);
    jLabel12.setText("From realm:");
    jLabel12.setPreferredSize(new Dimension(150, 17));
    jLabel12.setMinimumSize(new Dimension(150, 17));
    jLabel12.setMaximumSize(new Dimension(150, 17));
    realmPanel.setLayout(new java.awt.BorderLayout());
    realmPanel.setBounds(new java.awt.Rectangle(0, 0, 564, 300));
    realmPanel.add(realmPanel1, java.awt.BorderLayout.NORTH);
    mainPanel.setLayout(borderLayout1);
    propertiesPanel.setLayout(borderLayout2);
    jLabel1.setMaximumSize(new Dimension(150, 17));
    jLabel1.setMinimumSize(new Dimension(150, 17));
    jLabel1.setPreferredSize(new Dimension(150, 17));
    jLabel1.setText("Name:");
    jLabel2.setText("Comments:");
    jLabel2.setMinimumSize(new Dimension(150, 17));
    jLabel2.setMaximumSize(new Dimension(150, 17));
    jLabel2.setPreferredSize(new Dimension(150, 17));
    jLabel3.setPreferredSize(new Dimension(150, 17));
    jLabel3.setMaximumSize(new Dimension(150, 17));
    jLabel3.setMinimumSize(new Dimension(150, 17));
    jLabel3.setText("Specification:");
    jLabel4.setText("Specification language:");
    jLabel4.setPreferredSize(new Dimension(150, 17));
    jLabel4.setMinimumSize(new Dimension(150, 17));
    jLabel4.setMaximumSize(new Dimension(150, 17));
    namePanel.setLayout(flowLayout3);
    commentsPanel.setLayout(flowLayout4);
    specificationPanel.setLayout(flowLayout5);
    specificationLangPanel.setLayout(flowLayout6);
    jPanel1.setLayout(borderLayout3);
    jPanel2.setLayout(borderLayout4);
    jPanel3.setLayout(borderLayout5);
    typePanel.setLayout(borderLayout11);
    boundsPanel1.setLayout(flowLayout7);
    jLabel5.setMaximumSize(new Dimension(150, 17));
    jLabel5.setMinimumSize(new Dimension(150, 17));
    jLabel5.setPreferredSize(new Dimension(150, 17));
    jLabel5.setText("Type:");
    jLabel6.setMaximumSize(new Dimension(150, 17));
    jLabel6.setMinimumSize(new Dimension(150, 17));
    jLabel6.setPreferredSize(new Dimension(150, 17));
    jLabel6.setText("Multiplicity:");
    jLabel7.setMaximumSize(new Dimension(32767, 17));
    jLabel7.setMinimumSize(new Dimension(2, 17));
    jLabel7.setPreferredSize(new Dimension(40, 17));
    jLabel7.setHorizontalAlignment(SwingConstants.CENTER);
    jLabel7.setHorizontalTextPosition(SwingConstants.CENTER);
    jLabel7.setText("..");
    constraintPanel.setLayout(borderLayout6);
    jLabel8.setMaximumSize(new Dimension(150, 17));
    jLabel8.setMinimumSize(new Dimension(150, 17));
    jLabel8.setPreferredSize(new Dimension(150, 17));
    jLabel8.setText("Correctness constraint:");
    correctnessPanel.setLayout(flowLayout8);
    constraintPanel1.setLayout(borderLayout7);
    jLabel9.setText("Completeness constraint:");
    jLabel9.setPreferredSize(new Dimension(150, 17));
    jLabel9.setMinimumSize(new Dimension(150, 17));
    jLabel9.setMaximumSize(new Dimension(150, 17));
    completenessPanel.setLayout(flowLayout10);
    boundsPanel.setLayout(borderLayout10);
    typePanel1.setLayout(borderLayout14);
    iconOnlyPanel.setLayout(flowLayout2);
    typePanel2.setLayout(flowLayout12);
    compositeButtonPanel.setLayout(flowLayout13);
    solutionsBtn.setText("Solution patterns");
    solutionsBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        solutionsBtn_actionPerformed(e);
      }
    });
    nameEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    nameEdit.setPreferredSize(new Dimension(400, 21));
    jScrollPane1.setBorder(nameEdit.getBorder());
    jScrollPane1.setPreferredSize(new Dimension(400, 100));
    commentsEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    commentsEdit.setTabSize(2);
    jScrollPane2.setBorder(nameEdit.getBorder());
    jScrollPane2.setPreferredSize(new Dimension(400, 100));
    specificationLangEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    specificationLangEdit.setPreferredSize(new Dimension(400, 21));
    specificationEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    specificationEdit.setTabSize(2);
    borderLayout1.setHgap(5);
    borderLayout1.setVgap(5);
    typeEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    typeEdit.setPreferredSize(new Dimension(400, 21));
    correctnessBox.setBorder(nameEdit.getBorder());
    correctnessBox.setPreferredSize(new Dimension(400, 24));
    completenessBox.setBorder(nameEdit.getBorder());
    completenessBox.setPreferredSize(new Dimension(400, 24));
    lowerBoundBox.setBorder(nameEdit.getBorder());
    lowerBoundBox.setPreferredSize(new Dimension(175, 24));
    upperBoundBox.setBorder(nameEdit.getBorder());
    upperBoundBox.setPreferredSize(new Dimension(175, 24));
    iconOnlyCheckBox.setPreferredSize(new Dimension(250, 25));
    iconOnlyCheckBox.setText("Show icon only");
    mainPanel.setBorder(BorderFactory.createEtchedBorder());
    typePanel3.setLayout(borderLayout12);
    implementationsBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        implementationsBtn_actionPerformed(e);
      }
    });
    implementationsBtn.setText("Implementations");
    implementationsPanel1.setLayout(flowLayout14);
    implementationsPanel.setLayout(borderLayout13);
    implPattButton.setText("Implementation generators");
    implPattButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        implPattButton_actionPerformed(e);
      }
    });
    partOfPanel.setLayout(flowLayout9);
    partOfBox.setBorder(nameEdit.getBorder());
    partOfBox.setPreferredSize(new Dimension(400, 24));
    jLabel11.setText("Part of role:");
    jLabel11.setPreferredSize(new Dimension(150, 17));
    jLabel11.setMinimumSize(new Dimension(150, 17));
    jLabel11.setMaximumSize(new Dimension(150, 17));
    lastPanel.setLayout(borderLayout9);
    publicNamePanel.setLayout(flowLayout11);
    publicNameEdit.setFont(new java.awt.Font("Dialog", 0, 12));
    publicNameEdit.setPreferredSize(new Dimension(400, 21));
    jLabel10.setMaximumSize(new Dimension(150, 17));
    jLabel10.setMinimumSize(new Dimension(150, 17));
    jLabel10.setPreferredSize(new Dimension(150, 17));
    jLabel10.setText("Published as (name):");
    getContentPane().add(mainPanel);
    constraintPanel.add(constraintPanel1, BorderLayout.NORTH);
    constraintPanel1.add(correctnessPanel, BorderLayout.NORTH);
    correctnessPanel.add(jLabel8, null);
    correctnessPanel.add(correctnessBox, null);
    constraintPanel1.add(completenessPanel, BorderLayout.CENTER);
    completenessPanel.add(jLabel9, null);
    completenessPanel.add(completenessBox, null);
    buttonPanel.add(okBtn, null);
    buttonPanel.add(cancelBtn, null);
    mainPanel.add(propertiesPanel,  BorderLayout.NORTH);
    propertiesPanel.add(namePanel, BorderLayout.NORTH);
    namePanel.add(jLabel1, null);
    namePanel.add(nameEdit, null);
    propertiesPanel.add(jPanel1, BorderLayout.CENTER);
    jPanel1.add(commentsPanel, BorderLayout.NORTH);
    commentsPanel.add(jLabel2, null);
    commentsPanel.add(jScrollPane1, null);
    jPanel1.add(jPanel2, BorderLayout.CENTER);
    jPanel2.add(specificationPanel, BorderLayout.NORTH);
    specificationPanel.add(jLabel3, null);
    specificationPanel.add(jScrollPane2, null);
    jPanel2.add(jPanel3, BorderLayout.CENTER);
    jPanel3.add(specificationLangPanel,  BorderLayout.NORTH);
    specificationLangPanel.add(jLabel4, null);
    specificationLangPanel.add(specificationLangEdit, null);
    jPanel3.add(lastPanel, BorderLayout.CENTER);
    lastPanel.add(partOfPanel, BorderLayout.NORTH);
    partOfPanel.add(jLabel11, null);
    partOfPanel.add(partOfBox, null);
    jScrollPane2.getViewport().add(specificationEdit, null);
    jScrollPane1.getViewport().add(commentsEdit, null);
    typePanel.add(typePanel1, BorderLayout.NORTH);
    typePanel1.add(typePanel2, BorderLayout.NORTH);
    typePanel2.add(jLabel5, null);
    typePanel2.add(typeEdit, null);
    typePanel1.add(typePanel3, BorderLayout.CENTER);
    typePanel3.add(compositeButtonPanel, BorderLayout.CENTER);
    compositeButtonPanel.add(solutionsBtn, null);
    typePanel3.add(iconOnlyPanel, BorderLayout.NORTH);
    iconOnlyPanel.add(iconOnlyCheckBox, null);
    implementationsPanel.add(implementationsPanel1, BorderLayout.NORTH);
    implementationsPanel1.add(implementationsBtn, null);
    boundsPanel.add(boundsPanel1, BorderLayout.NORTH);
    boundsPanel1.add(jLabel6, null);
    boundsPanel1.add(lowerBoundBox, null);
    boundsPanel1.add(jLabel7, null);
    boundsPanel1.add(upperBoundBox, null);
    compositeButtonPanel.add(implPattButton, null);
    okBtn.setText("OK");
    okBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        okBtn_actionPerformed(e);
      }
    });
    cancelBtn.setText("Cancel");
    cancelBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        cancelBtn_actionPerformed(e);
      }
    });
    buttonPanel.setLayout(flowLayout1);
    mainPanel.add(buttonPanel, BorderLayout.SOUTH);
    publicNamePanel.add(jLabel10, null);
    publicNamePanel.add(publicNameEdit, null);
    realmPanel1.add(jLabel12);
    realmPanel1.add(fromRealmEdit);
  } //jbInit

  /* Manual initialization. */
  private void mInit()
  {
    File dir;

    // Stop right here if no concept is given.
    if (concept_ == null)
      return;

    // Fill all panels with data.
    // fill comboboxes for multiplicity bounds
    lowerBoundBox.addItem("0");
    lowerBoundBox.addItem("1");
    upperBoundBox.addItem("1");
    upperBoundBox.addItem("n");
    // fill correctness and completeness constraint combobox
    correctnessBox.addItem("<<none>>");
    completenessBox.addItem("<<none>>");
    dir = new File(CONSTRAINTS_DIR);
    String[] cfiles = dir.list(this);
    for (int fileIdx = 0; fileIdx < cfiles.length; fileIdx++)
    {
      // add pure class names
      correctnessBox.addItem(cfiles[fileIdx].substring(0,
        cfiles[fileIdx].length() - 6));
      completenessBox.addItem(cfiles[fileIdx].substring(0,
        cfiles[fileIdx].length() - 6));
    }
    // fill in partOf combobox
    partOfBox.addItem("<<none>>");
    Model model = concept_.getModel();
    if (model != null)
    {
      for (Node thisNode = model.firstNode(); thisNode != null;
        thisNode = model.nextNode(thisNode))
      {
        if (thisNode instanceof EDU.utwente.CoCompose.lang.SolutionRole)
        {
          partOfBox.addItem(thisNode.getLabel());
          // save in vector too, so we can retrieve the object later on.
          partOf_.add(thisNode);
        }
      }
    }

    // Flip in optional extra panels.
    addToPanel_ = mainPanel;
    if (concept_ instanceof EDU.utwente.CoCompose.lang.ConstrainedConcept)
    {
      addToPanel_.add(constraintPanel, BorderLayout.CENTER);
      addToPanel_ = constraintPanel;
    }
    if (concept_ instanceof EDU.utwente.CoCompose.lang.Concept)
    {
      addToPanel_.add(realmPanel, BorderLayout.CENTER);
      addToPanel_ = realmPanel;
      addToPanel_.add(implementationsPanel, BorderLayout.CENTER);
      addToPanel_ = implementationsPanel;
    }
    else if (concept_ instanceof EDU.utwente.CoCompose.lang.Composite)
    {
      addToPanel_.add(realmPanel, BorderLayout.CENTER);
      addToPanel_ = realmPanel;
      addToPanel_.add(typePanel, BorderLayout.CENTER);
      addToPanel_ = typePanel;
    }
    else if (concept_ instanceof EDU.utwente.CoCompose.lang.CompositeRole)
    {
      addToPanel_.add(boundsPanel, BorderLayout.CENTER);
      addToPanel_ = boundsPanel;
    }
    else if (concept_ instanceof EDU.utwente.CoCompose.lang.PublishedConcept)
    {
      addToPanel_.add(publicNamePanel, BorderLayout.CENTER);
      addToPanel_ = publicNamePanel;
    }
  } //mInit

  /* Reads properties from the concept, if any. */
  private void readProperties()
  {
    if (concept_ != null)
    {
      nameEdit.setText(concept_.getName());
      commentsEdit.setText(concept_.getComments());
      specificationEdit.setText(concept_.getSpecification());
      specificationLangEdit.setText(concept_.getSpecificationLang());
      if (concept_.getPartOf() != null)
        partOfBox.setSelectedItem(concept_.getPartOf().getLabel());
      if (concept_.getUniqueRealm() != null)
        fromRealmEdit.setText(concept_.getUniqueRealm().getName());
      // read class specific attributes
      if (concept_ instanceof EDU.utwente.CoCompose.lang.ConstrainedConcept)
      {
        correctnessBox.setSelectedItem(((ConstrainedConcept) concept_).getCorrectnessConstraint());
        completenessBox.setSelectedItem(((ConstrainedConcept) concept_).getCompletenessConstraint());
      } //if ConstrainedConcept
      if (concept_ instanceof EDU.utwente.CoCompose.lang.Composite)
      {
        typeEdit.setText(((EDU.utwente.CoCompose.lang.Composite) concept_).getType());
        iconOnlyCheckBox.setSelected(((EDU.utwente.CoCompose.lang.Composite) concept_).getShowIconOnly());
      } //if Composite
      else if (concept_ instanceof EDU.utwente.CoCompose.lang.CompositeRole)
      {
        lowerBoundBox.setSelectedIndex(
          ((CompositeRole) concept_).getMultLowerBound());
        upperBoundBox.setSelectedIndex(
          ((CompositeRole) concept_).getMultUpperBound() - 1);
      } //if CompositeRole
      else if (concept_ instanceof EDU.utwente.CoCompose.lang.PublishedConcept)
      {
        publicNameEdit.setText(((PublishedConcept) concept_).getPublicName());
      } //if PublishedConcept
    }
  } //readProperties

  /* Sets properties to the concept, if any. */
  private void setProperties()
  {
    try
    {
      concept_.setName(nameEdit.getText());
      concept_.setComments(commentsEdit.getText());
      concept_.setSpecification(specificationEdit.getText());
      concept_.setSpecificationLang(specificationLangEdit.getText());
      if (partOfBox.getSelectedIndex() > 0)
        concept_.setPartOf((SolutionRole) partOf_.get(
          partOfBox.getSelectedIndex() - 1));
      else
        concept_.setPartOf(null);
      // set class specific attributes
      if (concept_ instanceof EDU.utwente.CoCompose.lang.ConstrainedConcept)
      {
        if (correctnessBox.getSelectedIndex() > 0)
          ((ConstrainedConcept) concept_).setCorrectnessConstraint(
            (String) correctnessBox.getSelectedItem());
        else
          ((ConstrainedConcept) concept_).setCorrectnessConstraint("");
        if (completenessBox.getSelectedIndex() > 0)
          ((ConstrainedConcept) concept_).setCompletenessConstraint(
            (String) completenessBox.getSelectedItem());
        else
          ((ConstrainedConcept) concept_).setCompletenessConstraint("");
      } //if ConstrainedConcept
      if (concept_ instanceof EDU.utwente.CoCompose.lang.Composite)
      {
        if ((typeEdit.getText().length() != 0) ||
           (((EDU.utwente.CoCompose.lang.Composite) concept_).getType().length() != 0))
        {
          ((EDU.utwente.CoCompose.lang.Composite) concept_).setType(typeEdit.getText());
        }
        ((EDU.utwente.CoCompose.lang.Composite) concept_).setShowIconOnly(iconOnlyCheckBox.isSelected());
      } //if Composite
      else if (concept_ instanceof EDU.utwente.CoCompose.lang.CompositeRole)
      {
        ((CompositeRole) concept_).setMultLowerBound(
          lowerBoundBox.getSelectedIndex());
        ((CompositeRole) concept_).setMultUpperBound(
          upperBoundBox.getSelectedIndex() + 1);
      } //if CompositeRole
      else if (concept_ instanceof EDU.utwente.CoCompose.lang.PublishedConcept)
      {
        ((PublishedConcept) concept_).setPublicName(publicNameEdit.getText());
      } //if PublishedConcept
    }
    catch (CoComposeException e)
    {
      reportError("Couldn't set all properties: " + e.getMessage(), "Error",
        e.getConcept());
    }
    if (concept_ instanceof EDU.utwente.CoCompose.lang.ConstrainedConcept)
    {
      try
      {
        ((ConstrainedConcept) concept_).checkCorrectness();
      }
      catch (CoComposeException e)
      {
        reportError("The correctness constraints for \"" + concept_
          + "\" no longer apply: " + e.getMessage(), "Warning", e.getConcept());
      }
    }
  } //setProperties

  /* Close window. */
  void cancelBtn_actionPerformed(ActionEvent e)
  {
    dispose();
  } //cancelBtn_actionPerformed

  /* Apply new properties and close window. */
  void okBtn_actionPerformed(ActionEvent e)
  {
    setProperties();
    dispose();
  } //okBtn_actionPerformed

  /* Show solution patterns dialog. */
  void solutionsBtn_actionPerformed(ActionEvent e)
  {
    if (!(concept_ instanceof EDU.utwente.CoCompose.lang.Composite))
      return;

    SolPattDialog solutionsDlg = new SolPattDialog(frame_,
      "Edit solution patterns", true, (EDU.utwente.CoCompose.lang.Composite) concept_);
    solutionsDlg.show();
  } //solutionsBtn_actionPerformed

  /* Show implementation generators dialog. */
  void implPattButton_actionPerformed(ActionEvent e)
  {
    if (!(concept_ instanceof EDU.utwente.CoCompose.lang.Composite))
      return;

    ImplGenDialog implementationsDlg = new ImplGenDialog(frame_,
      "Edit implementation generators", true, (EDU.utwente.CoCompose.lang.Composite) concept_);
    implementationsDlg.show();
  } //implPattButton_actionPerformed

  /* Show implementations dialog. */
  void implementationsBtn_actionPerformed(ActionEvent e)
  {
    if (!(concept_ instanceof EDU.utwente.CoCompose.lang.Concept))
      return;

    ImplementationsDialog implementationsDlg = new ImplementationsDialog(frame_,
      "Edit implementations", true, (Concept) concept_);
    implementationsDlg.show();
  } //implementationsBtn_actionPerformed

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * File filter accepter - from interface FilenameFilter. Accepts .class files.
   *
   * @param dir The directory in which the file was found.
   * @param name The name of the file.
   * @return True if and only if the name should be included in the file list.
   */
  public boolean accept(File dir, String name)
  {
    // accept all class files
    return (name.endsWith(".class"));
  } //accept

  /**
   * Shows dialog window.
   */
  public void show()
  {
    readProperties();
    super.show();
  } //show
} //class ConceptPropertiesDialog
