/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.border.*;
import java.io.*;
import java.util.*;
import EDU.utwente.CoCompose.lang.*;
import EDU.utwente.CoCompose.util.*;

/**
 * Main window containing the main menu and composite repository. This window
 * starts up all model editor windows and any other windows.
 *
 * @author Dennis Wagelaar
 * @version 0.1.10, 22 July 2003
 */

public class MainFrame extends JFrame
  implements RepositoryContainer, ErrorReporter
{
  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  private int modelFrameCount_ = 0;
  private javax.swing.Timer statusTimeout_;
  ImageIcon appIcon = new ImageIcon("images/cocompose_icon.gif");
  ImageIcon aboutIcon = new ImageIcon("images/concept.gif");
  ImageIcon newIcon = new ImageIcon("images/new.gif");
  ImageIcon openIcon = new ImageIcon("images/open.gif");
  ImageIcon helpIcon = new ImageIcon("images/help.gif");
  ImageIcon exitIcon = new ImageIcon("images/door.gif");
  ImageIcon refreshIcon = new ImageIcon("images/refresh.gif");
  JMenuBar menuBar1 = new JMenuBar();
  JMenu menuFile = new JMenu();
  JMenuItem menuFileExit = new JMenuItem();
  JMenu menuHelp = new JMenu();
  JMenuItem menuHelpAbout = new JMenuItem();
  JLabel statusBar = new JLabel();
  BorderLayout borderLayout1 = new BorderLayout();
  CompoundBorder titledBorder1;
  CompoundBorder titledBorder2;
  JMenuItem menuFileNew = new JMenuItem();
  JMenuItem menuFileOpen = new JMenuItem();
  JPanel jPanel1 = new JPanel();
  JTree reposTree = new JTree(new Hashtable());
  BorderLayout borderLayout2 = new BorderLayout();
  JPanel jPanel2 = new JPanel();
  JButton newBtn = new JButton();
  JButton openBtn = new JButton();
  GridLayout gridLayout1 = new GridLayout();
  JPopupMenu treePopup = new JPopupMenu();
  JMenuItem menuRefreshTree = new JMenuItem();
  JMenuItem menuRepositoryRefresh = new JMenuItem();
  JMenuItem menuAPIHelp = new JMenuItem();
  JMenuItem menuToolHelp = new JMenuItem();
  JMenuItem MenuReadme = new JMenuItem();

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs the MainFrame.
   */
  public MainFrame()
  {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try
    {
      jbInit();
      mInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  } //MainFrame

  //////////////////////////////////////////////////////////////////////////////
  // Private methods
  //////////////////////////////////////////////////////////////////////////////

  /* JBuilder Component initialization. */
  private void jbInit() throws Exception
  {
    titledBorder1 = BorderFactory.createCompoundBorder(
      new TitledBorder(BorderFactory.createEtchedBorder(
        Color.white, new java.awt.Color(148, 145, 140)), "Main menu"),
          BorderFactory.createEmptyBorder(0,1,1,1));
    titledBorder2 = BorderFactory.createCompoundBorder(
      new TitledBorder(BorderFactory.createEtchedBorder(
        Color.white, new java.awt.Color(148, 145, 140)), "Composite Repository"),
          BorderFactory.createEmptyBorder(0,1,1,1));
    this.getContentPane().setLayout(borderLayout1);
    this.setSize(new Dimension(211, 670));
    this.setTitle("CoCompose");
    statusBar.setBorder(BorderFactory.createEtchedBorder());
    statusBar.setText("Ready");
    menuFile.setIcon(newIcon);
    menuFile.setText("File");
    menuFileExit.setIcon(exitIcon);
    menuFileExit.setText("Exit");
    menuFileExit.setAccelerator(javax.swing.KeyStroke.getKeyStroke(115, java.awt.event.KeyEvent.ALT_MASK, false));
    menuFileExit.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        fileExit_actionPerformed(e);
      }
    });
    menuHelp.setIcon(helpIcon);
    menuHelp.setText("Help");
    menuHelpAbout.setIcon(aboutIcon);
    menuHelpAbout.setText("About");
    menuHelpAbout.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        helpAbout_actionPerformed(e);
      }
    });
    menuFileOpen.setIcon(openIcon);
    menuFileOpen.setText("Open model");
    menuFileOpen.setAccelerator(javax.swing.KeyStroke.getKeyStroke(79, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuFileOpen.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuFileOpen_actionPerformed(e);
      }
    });
    menuFileNew.setIcon(newIcon);
    menuFileNew.setText("New model");
    menuFileNew.setAccelerator(javax.swing.KeyStroke.getKeyStroke(78, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuFileNew.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuFileNew_actionPerformed(e);
      }
    });
    jPanel1.setLayout(borderLayout2);
    reposTree.setBorder(BorderFactory.createLoweredBevelBorder());
    reposTree.setPreferredSize(new Dimension(200, 4));
    reposTree.addMouseListener(new java.awt.event.MouseAdapter()
    {

      public void mousePressed(MouseEvent e)
      {
        reposTree_mousePressed(e);
      }
    });
    jPanel1.setBorder(titledBorder2);
    newBtn.setHorizontalAlignment(SwingConstants.LEFT);
    newBtn.setIcon(newIcon);
    newBtn.setText("New model");
    newBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuFileNew_actionPerformed(e);
      }
    });
    openBtn.setIcon(openIcon);
    openBtn.setText("Open model");
    openBtn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuFileOpen_actionPerformed(e);
      }
    });
    openBtn.setToolTipText("Open File");
    openBtn.setHorizontalAlignment(SwingConstants.LEFT);
    jPanel2.setLayout(gridLayout1);
    gridLayout1.setColumns(1);
    gridLayout1.setRows(2);
    jPanel2.setBorder(titledBorder1);
    treePopup.setInvoker(reposTree);
    menuRefreshTree.setIcon(refreshIcon);
    menuRefreshTree.setText("Refresh");
    menuRefreshTree.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuRefreshTree_actionPerformed(e);
      }
    });
    menuRepositoryRefresh.setIcon(refreshIcon);
    menuRepositoryRefresh.setText("Refresh repository");
    menuRepositoryRefresh.setAccelerator(javax.swing.KeyStroke.getKeyStroke(82, java.awt.event.KeyEvent.CTRL_MASK, false));
    menuRepositoryRefresh.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        menuRefreshTree_actionPerformed(e);
      }
    });
    menuAPIHelp.setIcon(helpIcon);
    menuAPIHelp.setText("API Help");
    menuAPIHelp.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuAPIHelp_actionPerformed(e);
      }
    });
    menuToolHelp.setIcon(helpIcon);
    menuToolHelp.setText("Tool Help");
    menuToolHelp.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuToolHelp_actionPerformed(e);
      }
    });
    MenuReadme.setIcon(helpIcon);
    MenuReadme.setText("Readme");
    MenuReadme.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        menuReadme_actionPerformed(e);
      }
    });
    menuFile.add(menuFileNew);
    menuFile.add(menuFileOpen);
    menuFile.addSeparator();
    menuFile.add(menuRepositoryRefresh);
    menuFile.addSeparator();
    menuFile.add(menuFileExit);
    menuHelp.add(MenuReadme);
    menuHelp.add(menuToolHelp);
    menuHelp.add(menuAPIHelp);
    menuHelp.addSeparator();
    menuHelp.add(menuHelpAbout);
    menuBar1.add(menuFile);
    menuBar1.add(menuHelp);
    this.setJMenuBar(menuBar1);
    this.getContentPane().add(jPanel1, BorderLayout.CENTER);
    jPanel1.add(reposTree, BorderLayout.CENTER);
    this.getContentPane().add(statusBar, BorderLayout.SOUTH);
    this.getContentPane().add(jPanel2, BorderLayout.NORTH);
    jPanel2.add(newBtn, null);
    jPanel2.add(openBtn, null);
    treePopup.add(menuRefreshTree);
    statusTimeout_ = new javax.swing.Timer(2000, new ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        statusBar_timeout(e);
      }
    });
    statusTimeout_.setRepeats(false);
  } //jbInit

  /* Manual initialization. */
  private void mInit()
  {
    // set window icon
    setIconImage(appIcon.getImage());
    // initialize repository
    buildRepository();
    // set window height to 80% or minimum height
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension size = getSize();
    size.height = Math.max(size.height, (int) (screenSize.height * 0.8));
    setSize(size);
  } //mInit

  /* Construct repository tree for this directory. */
  private MutableTreeNode buildRepository(File dir)
  {
    MutableTreeNode childNode;
    String name;

    DefaultMutableTreeNode node = new DefaultMutableTreeNode(dir.getName());
    // get all .xml files in this directory
    DirFileFilter filter = new DirFileFilter(".xml");
    filter.setAcceptDirs(false);
    filter.setAcceptFiles(true);
    File files[] = dir.listFiles(filter);
    // get all directories in this directory
    filter.setAcceptDirs(true);
    filter.setAcceptFiles(false);
    File dirs[] = dir.listFiles(filter);
    // recurse into these directories
    for (int dirIdx = 0; dirIdx < dirs.length; dirIdx++)
    {
      childNode = buildRepository(dirs[dirIdx]);
      // only add child node if it isn't empty (hide empty dirs)
      if (childNode.getChildCount() > 0)
      {
        node.add(childNode);
      }
    }
    // add files from this level
    for (int fileIdx = 0; fileIdx < files.length; fileIdx++)
    {
      name = files[fileIdx].getName();
      name = name.substring(0, name.length() - 4);
      node.add(new DefaultMutableTreeNode(name));
    }
    return node;
  } //buildRepository(File dir)

  /* Bring up a model editor window. */
  private ModelFrame buildModelFrame()
  {
    return buildModelFrame(new Model());
  } //buildModelFrame()

  /* Bring up a model editor window. */
  private ModelFrame buildModelFrame(Model model_in)
  {
    Dimension frameSize = getSize();
    ModelFrame model_window = new ModelFrame(model_in);

    // Pad window to main tool window
    int cascade = modelFrameCount_ % 8;
    model_window.setLocation(frameSize.width + cascade * 20, cascade * 20);
    modelFrameCount_++;

    model_window.setTitle("CoCompose Design Model " + modelFrameCount_);
    model_window.setRepositoryContainer(this);

    model_window.pack();

    // Set window size
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension modelFrameSize = model_window.getSize();
    modelFrameSize.height = Math.max(modelFrameSize.height, frameSize.height);
    modelFrameSize.width = Math.min(modelFrameSize.height + 200,
      screenSize.width - frameSize.width);
    model_window.setSize(modelFrameSize);

    model_window.show();

    return model_window;
  } //buildModelFrame(Model model_in)

  /* File | Exit action performed. */
  void fileExit_actionPerformed(ActionEvent e)
  {
    System.exit(0);
  } //fileExit_actionPerformed

  /* Help | About action performed. */
  void helpAbout_actionPerformed(ActionEvent e)
  {
    setStatus("About");
    MainFrame_AboutBox dlg = new MainFrame_AboutBox(this);
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension dlgSize = dlg.getPreferredSize();
    if (dlgSize.height > screenSize.height)
      dlgSize.height = screenSize.height;
    if (dlgSize.width > screenSize.width)
      dlgSize.width = screenSize.width;
    dlg.setLocation((screenSize.width - dlgSize.width) / 2, (screenSize.height - dlgSize.height) / 2);
    dlg.setModal(true);
    dlg.show();
  } //helpAbout_actionPerformed

  /* Help | Tool Help action. Opens CoCompose tool help. */
  void menuToolHelp_actionPerformed(ActionEvent e)
  {
    File helpFile;
    TextViewDialog viewDlg;

    try
    {
      helpFile = new File("./help/index.html");
      viewDlg = new TextViewDialog(this, "CoCompose Tool Help", false);
      viewDlg.setPage(helpFile.toURL());
      viewDlg.show();
    }
    catch (Exception exc)
    {
      reportError(new CoComposeException("Could not open tool help file: " + exc.getMessage()));
    }
  } //menuToolHelp_actionPerformed

  /* Help | API Help action. Opens CoCompose API help. */
  void menuAPIHelp_actionPerformed(ActionEvent e)
  {
    File helpFile;
    TextViewDialog viewDlg;

    try
    {
      helpFile = new File("./doc/index.html");
      viewDlg = new TextViewDialog(this, "CoCompose API Help", false);
      viewDlg.setPage(helpFile.toURL());
      viewDlg.show();
    }
    catch (Exception exc)
    { // javadoc API not available, display alternative page
      try
      {
        helpFile = new File("./doc/no-index.html");
        viewDlg = new TextViewDialog(this, "CoCompose API Help", true);
        viewDlg.setPage(helpFile.toURL());
        viewDlg.show();
      }
      catch (Exception exc2)
      {
        reportError(new CoComposeException("Could not open API help file: " + exc2.getMessage()));
      }
    }
  } //menuAPIHelp_actionPerformed

  /* Menu | Readme action. Opens readme file. */
  void menuReadme_actionPerformed(ActionEvent e)
  {
    File helpFile;
    TextViewDialog viewDlg;

    try
    {
      helpFile = new File("./Readme.html");
      viewDlg = new TextViewDialog(this, "CoCompose Readme", false);
      viewDlg.setPage(helpFile.toURL());
      viewDlg.show();
    }
    catch (Exception exc)
    {
      reportError(new CoComposeException("Could not open readme file: " + exc.getMessage()));
    }
  } //menuReadme_actionPerformed

  /* Create new model frame. */
  void menuFileNew_actionPerformed(ActionEvent e)
  {
    setStatus("Creating new model...");
    buildModelFrame();
    setStatus("New model created");
  } //menuFileNew_actionPerformed

  /* Resets statusbar text. */
  void statusBar_timeout(ActionEvent e)
  {
    statusBar.setText("Ready");
  } //statusBar_timeout

  /* Refresh the repository tree. */
  void menuRefreshTree_actionPerformed(ActionEvent e)
  {
    buildRepository();
  } //menuRefreshTree_actionPerformed

  /* Mouse clicked on the repository tree. */
  void reposTree_mousePressed(MouseEvent e)
  {
    // Right mouse button clicked
    if (e.getModifiers() == MouseEvent.BUTTON3_MASK)
    {
      treePopup.show(reposTree, e.getX(), e.getY());
    }
  } //reposTree_mousePressed

  /* Opens a model and creates a new model frame that contains it. */
  void menuFileOpen_actionPerformed(ActionEvent e)
  {
    JFileChooser openDlg = new JFileChooser("./models");
    DirFileFilter filter = new DirFileFilter(".xml");
    try
    {
      // filter files on .xml
      openDlg.setFileFilter(filter);
      openDlg.setDialogTitle("Open model");
      if (openDlg.showOpenDialog(this) != JFileChooser.APPROVE_OPTION)
        return;
      File file = openDlg.getSelectedFile();
      // set status
      setStatus("Loading model...");
      // filename chosen - open this file
      String uri = file.getAbsolutePath();
      try
      {
        String name = file.getName();
        if (name.endsWith(".xml"))
          name = name.substring(0, name.length() - 4);
        Model model = new Model(uri, name);
        ModelFrame modelFrame = buildModelFrame(model);
        // set window title
        modelFrame.setTitle(name);
        // set status
        setStatus("Model loaded");
      }
      catch (Exception exc)
      {
        reportError(new CoComposeException("There was an error parsing the XML file: " +
          exc.getMessage()));
        // set status
        setStatus("Error loading model");
      }
    }
    catch (Exception exc)
    {
      reportError(new CoComposeException("It appears your VM does not allow file loading: " +
        exc.getMessage()));
    }
  } //menuFileOpen_actionPerformed

  //////////////////////////////////////////////////////////////////////////////
  // Protected methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Overridden event processing so we can exit on System Close.
   *
   * @param e The window event.
   */
  protected void processWindowEvent(WindowEvent e)
  {
    super.processWindowEvent(e);
    if(e.getID() == WindowEvent.WINDOW_CLOSING)
    {
      fileExit_actionPerformed(null);
    }
  } //processWindowEvent

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Construct repository tree from repository directory. Call this method
   * whenever something changes in the composite repository.
   */
  public void buildRepository()
  {
    File rootDir = new File("./repository/");
    TreeNode root = buildRepository(rootDir);
    reposTree.setModel(new DefaultTreeModel(root));
  } //buildRepository()

  /**
   * Gets the selected composite from the repository, if any.
   *
   * @return The selected Composite, null if none selected.
   */
  public EDU.utwente.CoCompose.lang.Composite getSelectedComposite()
  {
    EDU.utwente.CoCompose.lang.Composite selectedComposite = null;
    TreePath selectedPath = reposTree.getSelectionPath();

    if (selectedPath != null)
    {
      // set status
      setStatus("Loading composite from repository...");
      // get file path
      String filepath = "./";
      for (int i = 0; i < selectedPath.getPathCount(); i++)
      {
        if (i > 0)
          filepath += "/";
        filepath += selectedPath.getPathComponent(i);
      }
      File path = new File(filepath + ".xml");
      filepath = path.getAbsolutePath();
      // open xml file
      try
      {
        CompositeType compositeType = new CompositeType(filepath);
        selectedComposite = new EDU.utwente.CoCompose.lang.Composite();
        selectedComposite.copyAttributes(compositeType);
        // set status
        setStatus("Composite loaded from repository");
      }
      catch (Exception exc)
      {
        reportError(new CoComposeException("There was an error parsing the XML file: " +
          exc.getMessage()));
        // set status
        setStatus("Error loading composite from repository");
      }
    }
    return selectedComposite;
  } //getSelectedComposite

  /**
   * Reports an error.
   *
   * @param error - the error to report
   */
  public void reportError(CoComposeException error)
  {
    AbstractConcept concept = error.getConcept();
    String type = error.getType();
    String message = error.getMessage();
    if (concept != null)
      concept.setFillColor(Color.red);
    MessageDialog errDlg = new MessageDialog(this, type, true);
    // switch type
    if (type.equals("Error"))
      errDlg.setIcon(MessageDialog.ICON_ERROR);
    else if (type.equals("Warning"))
      errDlg.setIcon(MessageDialog.ICON_WARNING);
    else if (type.equals("Info"))
      errDlg.setIcon(MessageDialog.ICON_INFO);
    errDlg.setMessage(message);
    errDlg.show();
  } //reportError

  /**
   * Accepts generic notification message.
   *
   * @param sender The message sender.
   */
  public void notify(Object sender)
  {
  } //notify

  /**
   * Sets status bar text and updates window.
   *
   * @param status The status text.
   */
  public void setStatus(String status)
  {
    statusBar.setText(status);
    // force repaint
    Graphics g = getGraphics();
    paint(g);
    g.dispose();
    // restart status timer
    statusTimeout_.restart();
  } //setStatus
} //class MainFrame
