/**
 * Title:        CoCompose
 * Copyright:    Copyright (c) 2002, 2003
 * Author:       Dennis Wagelaar
 * Organisation: University of Twente, Vrije Universiteit Brussel
 * Description:  CoCompose Design Language Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package EDU.utwente.CoCompose.gui;

import java.awt.*;
import javax.swing.*;
import EDU.auburn.VGJ.util.DDimension;
import EDU.auburn.VGJ.util.DPoint;
import java.awt.event.*;
import javax.swing.event.*;

/**
 * A panel containing a ViewportScroller, along with a button for centering
 * the viewport.
 *
 * @author Dennis Wagelaar
 * @version 0.1.07, 11 March 2003
 */

public class ViewPortPanel extends JPanel
{
  //////////////////////////////////////////////////////////////////////////////
  // Attributes
  //////////////////////////////////////////////////////////////////////////////

  private ViewportScroller portScroller = new ViewportScroller();
  /**
   * Only one <code>ChangeEvent</code> is needed per instance since the
   * event's only state is the source property.  The source of events
   * generated is always "this".
   */
  protected transient ChangeEvent changeEvent;
  ImageIcon centerIcon = new ImageIcon("images/center.gif");
  JPanel portScrollerPanel = new JPanel();
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel portButtonPanel = new JPanel();
  JButton centerBtn = new JButton();
  FlowLayout flowLayout1 = new FlowLayout();
  FlowLayout flowLayout2 = new FlowLayout();

  //////////////////////////////////////////////////////////////////////////////
  // Constructors
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a ViewPortPanel.
   */
  public ViewPortPanel()
  {
    super();
    try
    {
      jbInit();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  } //ViewPortPanel

  //////////////////////////////////////////////////////////////////////////////
  // Private methods
  //////////////////////////////////////////////////////////////////////////////

  /* JBuilder Component initialization. */
  private void jbInit() throws Exception
  {
    this.setLayout(borderLayout1);
    centerBtn.setIcon(centerIcon);
    centerBtn.setText("Center");
    centerBtn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        centerBtn_actionPerformed(e);
      }
    });
    portScrollerPanel.setLayout(flowLayout1);
    portButtonPanel.setLayout(flowLayout2);
    portScroller.addChangeListener(new javax.swing.event.ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        portScroller_stateChanged(e);
      }
    });
    this.add(portScrollerPanel,  BorderLayout.CENTER);
    this.add(portButtonPanel,  BorderLayout.SOUTH);
    portButtonPanel.add(centerBtn, null);
    portScrollerPanel.add(portScroller, null);
  } //jbInit

  /* Center button pressed. */
  void centerBtn_actionPerformed(ActionEvent e)
  {
    DDimension port_dim = getPortSize();
    DDimension cont_dim = getContentSize();

    double x = (cont_dim.width - port_dim.width) / 2.0;
    double y = (cont_dim.height - port_dim.height) / 2.0;

    setOffset(new DPoint(x, y));
    // post event
    fireStateChanged();
  } //centerBtn_actionPerformed

  /* The state of the ViewportScroller has changed. */
  void portScroller_stateChanged(ChangeEvent e)
  {
    // forward event
    fireStateChanged();
  } //portScroller_stateChanged

  //////////////////////////////////////////////////////////////////////////////
  // Protected methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Notifies all listeners that have registered interest for
   * notification on this event type.  The event instance
   * is lazily created using the parameters passed into
   * the fire method.
   *
   * @see EventListenerList
   */
  protected void fireStateChanged()
  {
    // Guaranteed to return a non-null array
    Object[] listeners = listenerList.getListenerList();
    // Process the listeners last to first, notifying
    // those that are interested in this event
    for (int i = listeners.length - 2; i >= 0; i -= 2)
    {
      if (listeners[i] == ChangeListener.class)
      {
        // Lazily create the event:
        if (changeEvent == null)
          changeEvent = new ChangeEvent(this);
        ((ChangeListener) listeners[i + 1]).stateChanged(changeEvent);
      }
    }
  } //fireStateChanged

  //////////////////////////////////////////////////////////////////////////////
  // Public methods
  //////////////////////////////////////////////////////////////////////////////

  /**
   * Makes the component visible or invisible. Overrides Component.setVisible.
   *
   * @param aFlag True if and only if the component should be visible.
   */
  public void setVisible(boolean aFlag)
  {
    super.setVisible(aFlag);
    portScroller.setVisible(aFlag);
  } //setVisible

  /**
   * Gets the ViewportScroller size.
   *
   * @return The ViewportScroller size.
   */
  public Dimension getPortScrollerSize()
  {
    return portScroller.getSize();
  } //getPortScrollerSize

  /**
   * Sets the ViewportScroller size.
   *
   * @param size The ViewportScroller size.
   */
  public void setPortScrollerSize(Dimension size)
  {
    portScroller.setSize(size);
  } //setPortScrollerSize

  /**
   * Gets the ViewportScroller content size.
   *
   * @return The ViewportScroller content size.
   */
  public DDimension getContentSize()
  {
    return portScroller.getContentSize();
  } //getContentSize

  /**
   * Sets the ViewportScroller content size.
   *
   * @param size The ViewportScroller content size.
   */
  public void setContentSize(DDimension size)
  {
    portScroller.setContentSize(size);
  } //setContentSize

  /**
   * Gets the ViewportScroller viewport size.
   *
   * @return The ViewportScroller viewport size.
   */
  public DDimension getPortSize()
  {
    return portScroller.getPortSize();
  } //getPortSize

  /**
   * Sets the ViewportScroller viewport size.
   *
   * @param size The ViewportScroller viewport size.
   */
  public void setPortSize(DDimension size)
  {
    portScroller.setPortSize(size);
  } //setPortSize

  /**
   * Returns whether the mouse is dragging in the AngleControl.
   *
   * @return True if and only if the mouse is dragging in the AngleControl.
   */
  public boolean isDragging()
  {
    return portScroller.isDragging();
  } //isDragging

  /**
   * Gets the scrolling offset.
   *
   * @return The scrolling offset.
   */
  public DPoint getOffset()
  {
    return portScroller.getOffset();
  } //getOffset

  /**
   * Sets the scrolling offset.
   *
   * @param offset The scrolling offset.
   */
  public void setOffset(DPoint offset)
  {
    portScroller.setOffset(offset);
  } //setOffset

  /**
   * Adds a ChangeListener to the viewport scroller.
   *
   * @param l the listener to be added
   */
  public void addChangeListener(ChangeListener l)
  {
    listenerList.add(ChangeListener.class, l);
  } //addChangeListener

  /**
   * Removes a ChangeListener from the viewport scroller.
   *
   * @param l the listener to be removed
   */
  public void removeChangeListener(ChangeListener l)
  {
    listenerList.remove(ChangeListener.class, l);
  } //removeChangeListener
} //class ViewPortPanel
